import os
import time
import urllib
from calendar import monthrange

from netforce.model import Model,fields,get_model
from netforce.database import get_active_db, get_connection
from netforce.access import get_active_user, get_active_company, set_active_user
import netforce.config as config

class PrintLaborCost(Model):
    _name="clinic.print.wizard.labor.cost"
    _string="Report Wizard Labor Cost"
    _audit_log=True

    _fields={
        "date": fields.Date("Month"),
        "period_id": fields.Many2One("clinic.period.line", "Period", search=True),
        "date_from": fields.Date("From", required=True),
        "date_to": fields.Date("To", required=True),
        "cycle_id": fields.Many2One("clinic.cycle","Cycle"),
        "branch_id": fields.Many2One("clinic.branch","Branch"),
        "department_id": fields.Many2One("clinic.department","Department"),
        "staff_type": fields.Selection([["doctor","Doctor"],["nurse","Nurse"],["staff","Staff"]],"Type", required=True),
        'staff_id': fields.Many2One("clinic.staff","Staff"),
        'state': fields.Selection([['draft','Draft'],['waiting_doc','Waiting Document'],['done','Done']],'State',search=True),
        'company_id': fields.Many2One('company','Company'),
        'file_pdf1': fields.File("File_Labor_cost_sunmary"),
        'file_pdf2': fields.File("File_Labor_cost_details"),
        'file_pdf3': fields.File("File_Labor_cost_Sub_Deatils(Doctor)"),
        'file_pdf4': fields.File("File_Labor_cost_Sub_Deatils(Nurse)"),
    }

    def _get_date_from(self,context={}):
        year,month,day=time.strftime("%Y-%m-%d").split("-")
        #return '%s-%s-01'%(year,month)
        return '%s-%s-%s'%(year,month,day)

    def _get_date_to(self,context={}):
        year,month,day=time.strftime("%Y-%m-%d").split("-")
        weekday, total_day=monthrange(int(year), int(month))
        #return "%s-%s-%s"%(year,month,total_day)
        return "%s-%s-%s"%(year,month,day)

    def default_get(self,field_names=None,context={},**kw):
        defaults=context.get("defaults",{})
        date_from=defaults.get("date_from", self._get_date_from())
        date_to=defaults.get("date_to", self._get_date_to())
        print('defaults ', defaults)
        yearnow=date_from.split("-")[0]
        if len(get_model('clinic.period').search_browse([['name','=',yearnow]]))==0:
            period_id=None
        for period in get_model('clinic.period').search_browse([['name','=',yearnow]]):
            for line in period.lines:
                if line.state=='open':
                    period_id=line.id
                    date_from=line.date_start
                    date_to=line.date_stop
                    break
        res={
            'period_id': period_id,
            'date': time.strftime("%Y-%m-%d"),
            'date_from': date_from,
            'date_to': date_to,
            'state': 'draft',
            'company_id': get_active_company(),
        }
        return res
    
    def generate_report(self,ids,context={}):
        db=get_connection()
        user_id=get_active_user()
        set_active_user(1)
        if ids:
            obj=self.browse(ids)[0]
        else:
            return
        obj.write({
            'state': 'waiting_doc',
            'file_pdf1': None,
            'file_pdf2': None,
            'file_pdf3': None,
            #'file_pdf4': None,
        })
        db.commit()

        def load_report(fname,link):
            host=config.config['host']
            port=config.config['port']
            hostname='http://%s:%s/'%(host,port)
            link=hostname+link
            req=urllib.request.Request(link)
            response=urllib.request.urlopen(req)
            data=response.read()
            fpath=os.path.join('/tmp/wizard_lc',fname)
            open(fpath,"wb").write(data)

        dbname=get_active_db()
        fdir=os.path.join("static","db",dbname,"files")
        tmp_dir='/tmp/wizard_lc'
        print('majins',fdir)
        print('majins',tmp_dir)
        if not os.path.isdir(tmp_dir):
            os.system("mkdir %s"%(tmp_dir))
        #Create File Name
        fname1='%s_wizard_sunmary_%s_to_%s.pdf'%(obj.id,obj.date_from,obj.date_to)
        fname2='%s_wizard_details_%s_to_%s.pdf'%(obj.id,obj.date_from,obj.date_to)
        fname3='%s_wizard_sub_details_%s_to_%s(%s).pdf'%(obj.id,obj.date_from,obj.date_to,obj.staff_type)
        #fname3='%s_wizard_sub_details_%s_to_%s(docter).pdf'%(obj.id,obj.date_from,obj.date_to)
        #fname4='%s_wizard_sub_details_%s_to_%s(nurse).pdf'%(obj.id,obj.date_from,obj.date_to)
        defaults='&data=%s&period_id=%s&date_from=%s&date_to=%s&cycle_id=%s&branch_id=%s&department_id=%s&staff_type=%s'%(obj.date_from,obj.period_id.id,obj.date_from,obj.date_to,obj.cycle_id.id or None,obj.branch_id.id or None,obj.department_id.id or None,obj.staff_type or None)
        lc_id=get_model('clinic.report.wizard.labor.cost').create({})
        link1="report?convert=pdf&refer_id=%s&template=report_wizard_labor_cost_summary&model=clinic.report.wizard.labor.cost&type=report_odt2&method=get_report_wizard_summary%s&type_report=wizard&user=admin" % (lc_id,defaults)
        load_report(fname1,link1)
        link2="report?convert=pdf&refer_id=%s&template=report_wizard_labor_cost_details&model=clinic.report.wizard.labor.cost&type=report_odt2&method=get_report_wizard_details%s&type_report=wizard&user=admin" % (lc_id,defaults)
        load_report(fname2,link2)
        link3="report?convert=pdf&refer_id=%s&template=report_wizard_labor_cost_sub_details&model=clinic.report.wizard.labor.cost&type=report_odt2&method=get_report_wizard_sub_details%s&type_report=wizard&user=admin" % (lc_id,defaults)
        load_report(fname3,link3)
        #link4="report?convert=pdf&refer_id=%s&template=report_wizard_labor_cost_sub_details&model=clinic.report.wizard.labor.cost&type=report_odt2&method=get_report_wizard_sub_details%s&type_report=wizard&user=admin" % (lc_id,defaults+'&staff_type=nurse')
        #load_report(fname4,link4)

        obj.write({
            'file_pdf1': '%s' % (fname1),
            'file_pdf2': '%s' % (fname2),
            'file_pdf3': '%s' % (fname3),
            #'file_pdf4': '%s' % (fname4),
            'state': 'done',
        })
        os.chdir('%s'%fdir)
        os.system("mv /%s/%s* ."%(tmp_dir,obj.id))
        os.chdir('../../../../')
        set_active_user(user_id)
        vals={
            'to_id': get_active_user(),
            'subject': 'Generate Report',
            'body': """
                Generate labor cost report for successfully.
                <a href="/ui#name=clinic_print_wizard_labor_cost&mode=form&active_id=%s" class="msg-btn-ok">Open Print Wizard Labor Cost</a>
            """%(obj.id),
        }
        get_model("message").create(vals)
        db.commit()
        return {
            'next': {
                'name': 'clinic.print.wizard.labor.cost',
                'mode': 'form',
                'active_id': obj.id,
            },
        }

    def wkf_gen_report(self,context={},**kw):
        print("#"*80)
        trigger_ids=context.get("trigger_ids")
        if not trigger_ids:
            raise Exception("Missing trigger ids")
        print("trigger_ids",trigger_ids)
        self.generate_report(trigger_ids,context,**kw)

    def do_generate(self,ids,context={}):
        self.trigger(ids,"do_gen")

    def onchange_date(self,context={}):
        data=context['data']
        date=data['date']
        year,month,day=date.split("-")
        weekday, total_day=monthrange(int(year), int(month))
        data['date_from']="%s-%s-01"%(year,month)
        data['date_to']="%s-%s-%s"%(year,month,total_day)
        return data

    def onchange_branch(self,context={}):
        data=context['data']
        data['department_id']=None
        return data

    def onchange_from(self,context={}):
        data=context['data']
        data['date_to']=data['date_from']
        return data
    
    def onchange_period(self,context={}):   
        data=context['data']
        period_id=data['period_id']
        period=get_model('clinic.period.line').browse(period_id)
        data['date_from']=period.date_start
        data['date_to']=period.date_stop
        return data

PrintLaborCost.register()
