import time
from datetime import datetime
from calendar import monthrange
from netforce.model import Model,fields,get_model
from netforce.access import get_active_company, set_active_company, get_active_user, set_active_user
from netforce.database import get_connection

class ReportLaborCost(Model):
    _name="clinic.report.wizard.labor.cost"
    _string="Report Wizard Labor Cost"
    _transient=True

    _fields={
        "date": fields.Date("Month"),
        "period_id": fields.Many2One("clinic.period.line","Period"),
        "date_from": fields.Date("From", required=True),
        "date_to": fields.Date("To", required=True),
        "branch_id": fields.Many2One("clinic.branch","Branch", required=False),
        "cycle_id": fields.Many2One("clinic.cycle","Cycle"),
        "department_id": fields.Many2One("clinic.department","Department", required=False),
        "staff_type": fields.Selection([["doctor","Doctor"],["nurse","Nurse"],["staff","Staff"]],"Type"),
        "staff_id": fields.Many2One("clinic.staff","Staff")
    }

    def _get_date_from(self,context={}):
        year,month,day=time.strftime("%Y-%m-%d").split("-")
        #return '%s-%s-01'%(year,month)
        return '%s-%s-%s'%(year,month,day)

    def _get_date_to(self,context={}):
        year,month,day=time.strftime("%Y-%m-%d").split("-")
        weekday, total_day=monthrange(int(year), int(month))
        #return "%s-%s-%s"%(year,month,total_day)
        return "%s-%s-%s"%(year,month,day)

    def default_get(self,field_names=None,context={},**kw):
        defaults=context.get("defaults",{})
        date_from=defaults.get("date_from", self._get_date_from())
        date_to=defaults.get("date_to", self._get_date_to())
        print('defaults ', defaults)
        yearnow=date_from.split("-")[0]
        period_id=None
        for period in get_model('clinic.period').search_browse([[]]):
            for line in period.lines:
                if line.state=='open':
                    period_id=line.id
                    date_from=line.date_start
                    date_to=line.date_stop
                    break
        res={
            'period_id': period_id,
            'date': time.strftime("%Y-%m-%d"),
            'date_from': date_from,
            'date_to': date_to,
        }
        return res

    def wizard_report_summary(self,ids,context={}):
        obj=self.browse(ids)[0]
        return {
            "next": {
                "type": "report_odt2",
                "model": self._name,
                "template": "report_wizard_labor_cost_summary",
                "refer_id": obj.id,
                "method": "get_report_wizard_summary",
                "convert": "pdf",
            }
        }

    def wizard_report_details(self,ids,context={}):
        obj=self.browse(ids)[0]
        return {
            "next": {
                "type": "report_odt2",
                "model": self._name,
                "template": "report_wizard_labor_cost_details",
                "refer_id": obj.id,
                "method": "get_report_wizard_details",
                "convert": "pdf",
            }
        }

    def wizard_report_sub_details(self,ids,context={}):
        obj=self.browse(ids)[0]
        return {
            "next": {
                "type": "report_odt",
                "model": self._name,
                "template": "report_wizard_labor_cost_sub_details",
                "refer_id": obj.id,
                "method": "get_report_wizard_sub_details",
                "convert": "pdf",
                "date_from": obj.date_from,
            }
        }

    def get_report_data(self,ids,context={}):
        return

    def get_report_wizard_summary(self,ids,context={}):
        obj=[]
        if ids:
            if not context.get('type_report'):
                obj = self.browse(ids)[0]
                date = obj.date
                period_id = obj.period_id.id
                date_from = obj.date_from
                date_to = obj.date_to
                cycle_id = obj.cycle_id.id or None
                staff_id = obj.staff_id.id or None
                staff_type = obj.staff_type or None
                branch_id = obj.branch_id.id or None
                department_id = obj.department_id.id or None
            else:
                date=context.get('date')
                period_id=context.get('period_id')
                date_from=context.get('date_from')
                date_to=context.get('date_to')
                if context.get('staff_type')=='None':
                    staff_type=None
                else:
                    staff_type=context.get('staff_type')
                if context.get('staff_id')=='None':
                    staff_id=None
                else:
                    staff_id=context.get('staff_id')
                if context.get('cycle_id')=='None':
                    cycle_id=None
                else:
                    cycle_id=context.get('cycle_id')
                if context.get('branch_id')=='None':
                    branch_id=None
                else:
                    branch_id=context.get('branch_id')
                if context.get('department_id')=='None':
                    department_id=None
                else:
                    department_id=context.get('department_id')
        if context.get('user')=='admin':
            user_id=get_active_user()
            set_active_user(1)
        company_id=get_active_company()
        if not company_id:
            set_active_company(1)
            company_id=1
        company=get_model("company").browse(company_id)
        pages1=[]
        pages2=[]
        t1=datetime.now()
        lc_vals={
            'date': date_from,
            'period_id': period_id,
            'date_from': date_from,
            'date_to': date_to,
            'cycle_id': cycle_id or None,
            'branch_id': branch_id or None,
            'report_type': 'cross',
            'department_id': department_id or None,
            'report_type_wizard':'t',
        }
        lc_id=get_model('clinic.report.labor.cost').create({},context={'defaults':lc_vals,})
        lc_data=get_model('clinic.report.labor.cost').get_report_data([lc_id])
        #data['lc_data'].append(lc_data) # END STEP 1
        data={
            "comp_name":company.name,
            "date_from_1": date_from,
            "date_to_1": date_to,
            "dlines2": lc_data['dlines2'] or None,
            "nlines2": lc_data['nlines2'] or None,
            "ctlines2": lc_data['ctlines2'] or None,
        }
        pages1.append(data)
        t2=datetime.now()
        print('step 1 : ',t2-t1)

        #STEP 2
        staff_data=[]
        if staff_type:
            staff_data.append(staff_type)
        else:
            staff_data=['doctor','nurse']
        for s_line in staff_data:
            s_vals={
                'date': date_from,
                'date_from': date_from,
                'date_to': date_to,
                'staff_type': staff_type or s_line,
                'staff_id': staff_id or None,
                'cycle_id': cycle_id or None,
                'branch_id': branch_id or None,
                'department_id': department_id or None,
            }
            sum_id=get_model('clinic.report.labor.cost.summary').create({},context={
                'defaults':s_vals,
            })
            sum_data=get_model('clinic.report.labor.cost.summary').get_report_data([sum_id])
            if s_line=='doctor':
                sum_data.update({'staff_name':'ตารางหมอ'})
            elif s_line=='nurse':
                sum_data.update({'staff_name':'ตารางพยาบาล'})
            data={
                "comp_name":company.name,
                "date_from_1": date_from,
                "date_to_1": date_to,
                "emp_type": sum_data['staff_type'] or None,
                "dpts_txt": sum_data['dpts_txt'] or None,
                "items": sum_data['items'] or None,
                "total_lines_txt": sum_data['total_lines_txt'] or None,
            }
            pages2.append(data)
            t2=datetime.now()

        if context.get('user')=='admin':
            set_active_user(user_id)
        if pages1 and pages2:
            pages1[-1]['is_last_page']=True
            pages2[-1]['is_last_page']=True
        return {
            "pages1": pages1,
            "pages2": pages2,
        }

    def get_report_wizard_details(self,ids,context={}):
        obj=[]
        if ids:
            if not context.get('type_report'):
                obj = self.browse(ids)[0]
                date = obj.date
                period_id = obj.period_id.id
                date_from = obj.date_from
                date_to = obj.date_to
                cycle_id = obj.cycle_id.id or None
                staff_id = obj.staff_id.id or None
                staff_type = obj.staff_type or None
                branch_id = obj.branch_id.id or None
                department_id = obj.department_id.id or None
            else:
                date=context.get('date')
                period_id=context.get('period_id')
                date_from=context.get('date_from')
                date_to=context.get('date_to')
                if context.get('staff_type')=='None':
                    staff_type=None
                else:
                    staff_type=context.get('staff_type')
                if context.get('staff_id')=='None':
                    staff_id=None
                else:
                    staff_id=context.get('staff_id')
                if context.get('cycle_id')=='None':
                    cycle_id=None
                else:
                    cycle_id=context.get('cycle_id')
                if context.get('branch_id')=='None':
                    branch_id=None
                else:
                    branch_id=context.get('branch_id')
                if context.get('department_id')=='None':
                    department_id=None
                else:
                    department_id=context.get('department_id')
        if context.get('user')=='admin':
            user_id=get_active_user()
            set_active_user(1)
        company_id=get_active_company()
        if not company_id:
            set_active_company(1)
            company_id=1
        company=get_model("company").browse(company_id)
        pages=[]
        t1=datetime.now()

        #STEP 2
        i=0
        staff_data=[]
        if staff_type:
            staff_data.append(staff_type)
        else:
            staff_data=['doctor','nurse']
        for s_line in staff_data:
            sum_id=get_model('clinic.report.labor.cost.summary').create({
                'date': date_from,
                'date_from': date_from,
                'date_to': date_to,
                'staff_type': staff_type or s_line,
                'staff_id': staff_id or None,
                'cycle_id': cycle_id or None,
                'branch_id': branch_id or None,
                'department_id': department_id or None,
            })
            sum_data=get_model('clinic.report.labor.cost.summary').get_report_data([sum_id])
            t2=datetime.now()

            #STEP 3
            for detail_line in sum_data['lines']:
                detail_id=get_model('clinic.report.labor.cost.detail').create({
                    'date': date_from,
                    'date_from': date_from,
                    'date_to': date_to,
                    'staff_type': detail_line['staff_type'] or None,
                    'staff_id': detail_line['staff_id'] or None,
                    'cycle_id': None,
                    'branch_id': branch_id or None,
                    'department_id': department_id or None,
                })
                detail_data=get_model('clinic.report.labor.cost.detail').get_report_data([detail_id])
                data={
                    "date_from_1": date_from,
                    "date_to_1": date_to,
                    "emp_type": detail_line['staff_type'] or None,
                    "emp_name": detail_line['staff_name'] or None,
                    "detail_dpts_txt": detail_data['dpts_txt'] or None,
                    "detail_items": detail_data['items'] or None,
                    "detail_total_lines_txt": detail_data['total_lines_txt'] or None,
                }
                pages.append(data)
                t2=datetime.now()
                i=i+1
                print('step 3 : ',t2-t1,' item in %d and name %s >>>>>>>> %s'%(i,detail_line['staff_name'],detail_line['staff_type']))
        if pages:
            pages[-1]['is_last_page']=True
        if context.get('user')=='admin':
            set_active_user(user_id)
        if not company_id:
            set_active_company(company_id)
        return {
            "pages": pages,
        }

    def get_report_wizard_sub_details(self,ids,context={}):
        obj=[]
        if ids:
            if not context.get('type_report'):
                obj = self.browse(ids)[0]
                date = obj.date
                period_id = obj.period_id.id
                date_from = obj.date_from
                date_to = obj.date_to
                cycle_id = obj.cycle_id.id or None
                staff_id = obj.staff_id.id or None
                staff_type = obj.staff_type or None
                branch_id = obj.branch_id.id or None
                department_id = obj.department_id.id or None
            else:
                date=context.get('date')
                period_id=context.get('period_id')
                date_from=context.get('date_from')
                date_to=context.get('date_to')
                if context.get('staff_type')=='None':
                    staff_type=None
                else:
                    staff_type=context.get('staff_type')
                if context.get('staff_id')=='None':
                    staff_id=None
                else:
                    staff_id=context.get('staff_id')
                if context.get('cycle_id')=='None':
                    cycle_id=None
                else:
                    cycle_id=context.get('cycle_id')
                if context.get('branch_id')=='None':
                    branch_id=None
                else:
                    branch_id=context.get('branch_id')
                if context.get('department_id')=='None':
                    department_id=None
                else:
                    department_id=context.get('department_id')
        if context.get('user')=='admin':
            user_id=get_active_user()
            set_active_user(1)
        company_id=get_active_company()
        if not company_id:
            set_active_company(1)
            company_id=1
        company=get_model("company").browse(company_id)
        pages=[]
        t1=datetime.now()

        staff_data=[]
        if staff_type:
            staff_data.append(staff_type)
        else:
            staff_data=['doctor']
        for s_line in staff_data:
            ctx_sum={
                'date': date_from,
                'date_from': date_from,
                'date_to': date_to,
                'staff_type': staff_type or s_line,
                'staff_id': None,
                'cycle_id': None,
                'branch_id': None,
                'department_id': None,

            }
            sum_id=get_model('clinic.report.labor.cost.summary').create({
                'date': date_from,
                'date_from': date_from,
                'date_to': date_to,
                'staff_type': staff_type or s_line,
                'staff_id': staff_id or None,
                'cycle_id': cycle_id or None,
                'branch_id': branch_id or None,
                'department_id': department_id or None,
            }, context=ctx_sum)
            sum_data=get_model('clinic.report.labor.cost.summary').get_report_data([sum_id])
            t2=datetime.now()
            for line in sum_data['lines']:
                ctx={
                    'defaults': {
                        'date': date_from,
                        'date_from': date_from,
                        'date_to': date_to,
                        'staff_type': line['staff_type'] or None,
                        'staff_id': line['staff_id'] or None,
                        'branch_id': branch_id or None,
                        'department_id': department_id or None,
                    }
                }
                sub_detail_id=get_model('clinic.report.labor.cost.sub.detail').create({},context=ctx)
                sub_data=get_model('clinic.report.labor.cost.sub.detail').get_report_data([sub_detail_id])
                data={
                    "date_from_1": date_from,
                    "date_to_1": date_to,
                    "emp_name": line['staff_name'] or None,
                    "emp_type": line['staff_type'] or None,
                    "lines": sub_data['lines'] or None,
                    "amount_total": line['total'] or None,
                }
                print('Get Data : Name : %s, Type : %s, Total : %s'%(line['staff_name'],line['staff_type'],line['total']))
                pages.append(data)
                t2=datetime.now
        if pages:
            pages[-1]['is_last_page']=True
        if context.get('user')=='admin':
            set_active_user(user_id)
        if not company_id:
            set_active_company(company_id)
        return {
            "pages":pages,
        }

    def onchange_date(self,context={}):
        data=context['data']
        date=data['date']
        year,month,day=date.split("-")
        weekday, total_day=monthrange(int(year), int(month))
        data['date_from']="%s-%s-01"%(year,month)
        data['date_to']="%s-%s-%s"%(year,month,total_day)
        return data

    def onchange_branch(self,context={}):
        data=context['data']
        data['department_id']=None
        return data

    def onchange_from(self,context={}):
        data=context['data']
        data['date_to']=data['date_from']
        return data

    def onchange_period(self,context={}):   
        data=context['data']
        period_id=data['period_id']
        period=get_model('clinic.period.line').browse(period_id)
        data['date_from']=period.date_start
        data['date_to']=period.date_stop
        return data

    def onchange_type(self,context={}):
        data=context['data']
        data['staff_id']=None
        return data

ReportLaborCost.register()
